import matplotlib.cm
from PIL import Image

from mandelbrot_03 import MandelbrotSet
from viewport import Viewport

resolution = 4096

def paint(mandelbrot_set, viewport, palette, smooth):
    for pixel in viewport:
        stability = mandelbrot_set.stability(complex(pixel), smooth)
        index = int(min(stability * len(palette), len(palette) - 1))
        pixel.color = palette[index % len(palette)]


def denormalize(palette):
    return [
        tuple(int(channel * 255) for channel in color) for color in palette
    ]


if __name__ == "__main__":
    print("This might take a while...")

    colormap = matplotlib.colormaps.get_cmap("twilight").colors
    palette = denormalize(colormap)

    mandelbrot_set = MandelbrotSet(max_iterations=512, escape_radius=1000)
    image = Image.new(mode="RGB", size=(resolution, resolution))
    viewport = Viewport(image, center=-0.7435 + 0.1314j, width=0.002)
    paint(mandelbrot_set, viewport, palette, smooth=True)
    image.show()
